#ifndef FUNCTIONS_H_INCLUDED
#define FUNCTIONS_H_INCLUDED

#include <string>
#include <cstdlib>

using namespace std;

/// Computs sgn(b) * |a|
inline Real SIGN(Real a, Real b) {
	if (b>=0.0) return fabs(a);
	return -fabs(a);
}

/// Min of two generic types
template <class T>
inline T MIN(T a, T b) {
	if (a<b) return a;
	return b;
}

/// Max of two generic types
template <class T>
inline T MAX(T a, T b) {
	if (a>=b) return a;
	else return b;
}

/// Returns a^2 = a*a
inline Real SQR(Real a) {
	return a*a;
}

/// Exchanges two values for a generic type
template <class T>
inline void SWAP(T& a, T& b) {
	T tmp=a;
	a=b;
	b=tmp;
	return;
}

/// Computes sqrt(a^2+b2) without destructive underflow or overflow
inline Real pythag(Real a, Real b) {
	double absa=fabs(a), absb=fabs(b);
	return (absa > absb ? absa*sqrt(1.0+SQR(absb/absa)) :
	(absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb))));
}

/// Returns a^n
inline Real puiss(Real a, U_Int n) {
	if (n==0) return 1.0;
	Real result=1.0;
	for (int i=n-1; i>=0; i--)
		result *= a;
		return result;
}

/// Check if an endline is reached : either CR (ASCII : 13) or line.end()
inline bool isEndLine(string::iterator& it,string& line) {
	return ( it==line.end() || *it==13 );
}

#endif // Functions
