#ifndef RANDOMGEN_H_INCLUDED
#define RANDOMGEN_H_INCLUDED

#include <cstdlib>
#include "BasicTypes.hpp"

/// A class to generate random numbers
/** Function Ran() from Numerical Recipes.
 * The period of the generator is around 3.138 10^57
 */
class Random {

public :

    static U_Long u,v,w;

    Random () {}

    /// Constructor ; call with any integer seed (except 4101842887655102017, see code).
    static void initialize (U_Long seed);

    ~Random () {}

    /// Returns a 64-bit random integer
    static inline U_Long rand64() {
        u = u * 2862933555777941757LL + 7046029254386353087LL;
        v ^= v >> 17; v ^= v << 31; v ^= v >> 8;
        w = 4294957665U*(w & 0xffffffff) + (w >> 32);
        U_Long x = u ^ (u << 21); x ^= x >> 35; x ^= x << 4;
        return (x + v) ^ w;
    }

    /// Returns a random double-precision floating value in the range 0. to 1.
    static inline Real rand01() {
        return 5.42101086242752217E-20 * rand64();
    }

    /// Returns a 32-bit random integer
    static inline U_Int rand32() {
        return (U_Int)rand64();
    }

	/// Returns +1 or -1 with probability 0.5
    static inline Int randSgn() {
        return 1 - 2 * (rand64()%2);
    }

    /// Returns a random double-precision floating value in the range -1. to 1.
    static inline Real rand_11() {
        return randSgn() * rand01();
    }
};

#define rand01 Random::rand01
#define rand_11 Random::rand_11
#define rand32 Random::rand32
#define rand64 Random::rand64
#define randSgn Random::randSgn

#endif // RandomGen
