#ifndef VECTEUR_H_INCLUDED
#define VECTEUR_H_INCLUDED

#include "AllUsefulHeaders.hpp"
#include "Column.hpp"

template <class T>
class MyVector : public vector<T> {

public :

	MyVector () {}

	~MyVector () {}

	// only modification needed :
	// push_back return true if reallocation
	bool addElem(T& t) {
		if (this->empty()) {
			this->push_back(t);
			return true;
		}

		T* ctrlAd = &((*this)[0]);
		this->push_back(t);
		if (&((*this)[0]) == ctrlAd) return false;
		return true;
	}
};

class Vecteur {

private :

	Real* elems; ///< elements of the vector/function

public :

    Int nbElems; ///< number of elements

    // check emptiness
    inline bool empty() {
        return (nbElems==0);
    }


    /*****************************
    * constructors, destructors
    ******************************/

    Vecteur () {
        elems=NULL;
        nbElems=0;
    }

    // initialize to zero
    Vecteur (Int nbElems) {
        elems=(Real*)calloc(nbElems,sizeof(Real));
        this->nbElems=nbElems;
    }

    /// copy constructor, sometimes truncated to k first elements
    Vecteur (const Vecteur& v_f, Int nbElems=0) {
        if (nbElems>0) {
            elems=(Real*)malloc(nbElems*sizeof(Real));
            this->nbElems=nbElems;
        }
        else {
            elems=(Real*)malloc(v_f.nbElems*sizeof(Real));
            this->nbElems=v_f.nbElems;
        }
        for (Int i=this->nbElems-1; i>=0; i--)
            elems[i]=v_f.elems[i];
    }

    /// very special constructor, from a column
    Vecteur (const Column& col) {
    	nbElems=col.size();
    	elems=(Real*)malloc(nbElems*sizeof(Real));
    	Int countElems=0;
    	for (list<Real*>::const_iterator it=col.begin(); it!=col.end(); it++)
			elems[countElems++] = *(*it);
    }

    ~Vecteur () {
        if (elems!=NULL) free(elems);
    }


    /*****************************
    * operators definitions
    ******************************/

    inline Real& operator[](Int i) {
        return elems[i];
    }

    // = operator, we realloc memory only when size change
    inline void operator=(Vecteur v_f) {
        if (nbElems!=v_f.nbElems) {
            if (elems!=NULL) free(elems);
            elems=(Real*)malloc(v_f.nbElems*sizeof(Real));
            nbElems=v_f.nbElems;
        }
        for (Int i=nbElems-1; i>=0; i--)
            elems[i]=v_f[i];
    }

    // assuption : same sizes (because special operation)
    inline void operator=(Column& col) {
        Int countElems=0;
        for (list<Real*>::iterator it=col.begin(); it!=col.end(); it++)
            elems[countElems++] = *(*it);
    }

    inline Vecteur operator+(Vecteur v_f) {
        Vecteur result(nbElems);
        for (Int i=nbElems-1; i>=0; i--)
            result[i]=elems[i] + v_f[i];
        return result;
    }

    inline Vecteur operator-(Vecteur v_f) {
        Vecteur result(nbElems);
        for (Int i=nbElems-1; i>=0; i--)
            result[i] = elems[i] - v_f[i];
        return result;
    }

    inline void operator+=(Vecteur& v_f) {
        for (Int i=nbElems-1; i>=0; i--)
            elems[i] += v_f[i];
    }

    inline void operator-=(Vecteur& v_f) {
        for (Int i=nbElems-1; i>=0; i--)
            elems[i] -= v_f[i];
    }

    // product with a scalar
    inline Vecteur operator*(Real scal) {
        Vecteur result(nbElems);
        for (Int i=nbElems-1; i>=0; i--)
            result[i] = scal * elems[i];
		return result;
    }

    inline void operator*=(Real scal) {
        for (Int i=nbElems-1; i>=0; i--)
            elems[i] *= scal;
    }

    // divide all elements by a scalar (supposed != 0.0)
    inline void operator/=(Real scal) {
        for (Int i=nbElems-1; i>=0; i--)
            elems[i] /= scal;
    }

    // zero all entries
    inline void zero() {
        for (Int i=nbElems-1; i>=0; i--)
            elems[i]=0.0;
    }

    /**********************
    * Fonctions diverses
    **********************/

    // sum all the elements (make sense only for Vecteur type)
    inline Real sum() {
        Real result=0.0;
        for (int i=nbElems-1; i>=0; i--)
            result += elems[i];
        return result;
    }

    // average VectOrFunc value
    inline Real mean() {
        if (empty()) return 0.0;
        return sum() / nbElems;
    }

    ////////////////////////////////
    // EUCLIDIAN :
    ////////////////////////////////

    // euclidian square norm :
    inline Real sq_norm2() {
        if (empty()) return 0.0;
        Real result=0.0;

        for (Int i=nbElems-1; i>=0; i--)
            result += elems[i] * elems[i];
        return result;
    }

    // "averaged" euclidian distance from a composant to 0.0
    inline Real norm2() {
        return sqrt(sq_norm2());
    }

    // normalize to 1 for the "average"square norm
    inline void normalize2() {
        (*this) /= norm2();
    }

    // return the "averaged" squared euclidian distance from this to v_f :
    inline Real sq_distance2(Vecteur& v_f) {
        Vecteur tmpDiff = *this - v_f;
        return tmpDiff.sq_norm2();
    }

    // return the "averaged" euclidian distance from this to v_f :
    inline Real distance2(Vecteur& v_f) {
        Vecteur tmpDiff = *this - v_f;
        return tmpDiff.norm2();
    }


	// "infinity" norm :
    inline Real normInf() {
        Real newVal, valNorm = - INFTY_REAL;
        for (Int i=nbElems-1; i>=0; i--) {
            if ( (newVal = fabs(elems[i])) > valNorm)
                valNorm = newVal;
        }
        return valNorm;
    }

    // scalar product : integral approximation in case of functions
    inline Real operator*(Vecteur& v_f) {
        if (empty()) return 0.0;
        Real result=0.0;

        for (Int i=nbElems-1; i>=0; i--)
            result += elems[i] * v_f[i];
        return result;
    }

    // only the Vecteur type can multiply a Column
    inline Real operator*(Column& col) {
        if (empty()) return 0.0;
        Real result=0.0;

        Int countElems=0;
        for (Column::iterator it=col.begin(); it!=col.end(); it++)
            result += elems[countElems++] * *(*it);

        return result;
    }

    // no assumption on v_f :
    void removeProjOnto(Vecteur& v_f) {
        Real scProd = (*this) * v_f;
        scProd /= v_f.sq_norm2();
        Vecteur tmpVect = v_f * scProd;
        *this -= tmpVect;
    }

    inline void scale(Vecteur& meanVect, Vecteur& stdVect) {
    	for (Int i=nbElems-1; i>=0; i--)
			elems[i] = (elems[i] - meanVect[i]) / stdVect[i];
	}
	inline void deScale(Vecteur& meanVect, Vecteur& stdVect) {
		for (Int i=nbElems-1; i>=0; i--)
			elems[i] = (stdVect[i] * elems[i])+ meanVect[i];
	}

	/**************************
	* read / write to file :
	**************************/

	/// write Vecteur to file
	void operator>>(ofstream& outFile) {
		// <value> <separator> <value> <separator> ... <value>
		for (Int i=0; i<(Int)nbElems; i++) {
			outFile << elems[i] << " ";
		}
		outFile << std::endl;
	}

	/// read Vecteur from file (for any separator)
	void operator<<(ifstream& inFile) {
		string line;
		getline(inFile,line,'\n');
		// parse the line to get only numerics :
		vector<Real> tmpValues;
		string::iterator it1=line.begin(), it2;
		while (!isEndLine(it1,line)) {
			// put it1 on the next readable digit
			while (!isEndLine(it1,line)
					&& !(*it1=='.' || *it1=='e' || *it1=='E' || *it1=='-'
					|| (*it1 >= '0' && *it1 <= '9')) )
				it1++;
			// put it2 on next separator :
			it2=it1;
			while (!isEndLine(it2,line)
					&& (*it2=='.' || *it2=='e' || *it2=='E' || *it2=='-'
					|| (*it2 >= '0' && *it2 <= '9')) )
				it2++;
			// get the corresponding value :
			string newValue(it1,it2);
			if (!newValue.empty()) tmpValues.push_back(atof(newValue.c_str()));
			// reinitialize it1 to the first next separator character :
			it1=it2;
		}
		if (!tmpValues.empty()) {
				nbElems = tmpValues.size();
        elems=(Real*)calloc(nbElems,sizeof(Real));
			for (Int i=0; i<tmpValues.size(); i++)
				elems[i] = tmpValues[i];
		}
	}
};

#endif // Vecteur
